==> gr-wxgui-0.1.tar.gz: gr-wxgui-0.1/src/python/gnuradio/wxgui/fftsink.py <==
#!/usr/bin/env python
#
# Copyright 2003,2004 Free Software Foundation, Inc.
# 
# This file is part of GNU Radio
# 
# GNU Radio is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
# 
# GNU Radio is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with GNU Radio; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.
# 

from gnuradio import gr
from gnuradio.wxgui import stdgui
import wx
import wx.lib.plot as plot
import Numeric
import os
import threading
    


# FIXME this should be rewritten to use hierarchical modules (when they're ready)

# ========================================================================
# returns (block, win).
#   block requires a single input stream of float
#   win is a subclass of wxWindow

def make_fft_sink_f (fg, parent, label, fft_size, input_rate):
    (r_fd, w_fd) = os.pipe ()
    fft_rate = 20

    s2p = gr.serial_to_parallel (gr.sizeof_float, fft_size)
    one_in_n = gr.keep_one_in_n (gr.sizeof_float * fft_size,
                             int (input_rate / fft_size / fft_rate))
    fft = gr.fft_vfc (fft_size, True, True)
    dst = gr.file_descriptor_sink (gr.sizeof_gr_complex * fft_size, w_fd)

    fg.connect (s2p, one_in_n)
    fg.connect (one_in_n, fft)
    fg.connect (fft, dst)

    block = s2p                       # head of pipeline

    win = fft_window (fft_info (r_fd, fft_size, input_rate, True, label), parent)
    
    return (block, win)

# ========================================================================
# returns (block, win).
#   block requires a single input stream of gr_complex
#   win is a subclass of wxWindow

def make_fft_sink_c (fg, parent, label, fft_size, input_rate):
    (r_fd, w_fd) = os.pipe ()
    fft_rate = 20

    s2p = gr.serial_to_parallel (gr.sizeof_gr_complex, fft_size)
    one_in_n = gr.keep_one_in_n (gr.sizeof_gr_complex * fft_size,
                             int (input_rate / fft_size / fft_rate))
    fft = gr.fft_vcc (fft_size, True, True)
    dst = gr.file_descriptor_sink (gr.sizeof_gr_complex * fft_size, w_fd)

    fg.connect (s2p, one_in_n)
    fg.connect (one_in_n, fft)
    fg.connect (fft, dst)

    block = s2p                       # head of pipeline

    win = fft_window (fft_info (r_fd, fft_size, input_rate, False, label), parent)

    return (block, win)

# ------------------------------------------------------------------------

myDATA_EVENT = wx.NewEventType()
EVT_DATA_EVENT = wx.PyEventBinder (myDATA_EVENT, 0)


class DataEvent(wx.PyEvent):
    def __init__(self, data):
        wx.PyEvent.__init__(self)
        self.SetEventType (myDATA_EVENT)
        self.data = data

    def Clone (self): 
        self.__class__ (self.GetId())


class fft_info:
    def __init__ (self, file_descriptor, fft_size, sample_rate, input_is_real, title = "fft"):
        self.file_descriptor = file_descriptor
        self.fft_size = fft_size
        self.sample_rate = sample_rate
        self.input_is_real = input_is_real
        self.title = title;
        
class input_watcher (threading.Thread):
    def __init__ (self, file_descriptor, fft_size, event_receiver, **kwds):
        threading.Thread.__init__ (self, **kwds)
        self.setDaemon (1)
        self.file_descriptor = file_descriptor
        self.fft_size = fft_size
        self.event_receiver = event_receiver
        self.keep_running = True
        self.start ()

    def run (self):
        # print "input_watcher: pid = ", os.getpid ()
        while (self.keep_running):
            s = os.read (self.file_descriptor, gr.sizeof_gr_complex * self.fft_size)
            if not s:
                self.keep_running = False
                break

            complex_data = Numeric.fromstring (s, Numeric.Complex32)
            de = DataEvent (complex_data)
            wx.PostEvent (self.event_receiver, de)
            # print "run: len(complex_data) = ", len(complex_data)
            del de
    

class fft_window (plot.PlotCanvas):
    def __init__ (self, info, parent, id = -1,
                  pos = wx.DefaultPosition, size = wx.DefaultSize,
                  style = wx.DEFAULT_FRAME_STYLE, name = ""):
        plot.PlotCanvas.__init__ (self, parent, id, pos, size, style, name)

        self.SetEnableGrid (True)
        self.SetEnableZoom (True)
        # self.SetBackgroundColour ('black')
        
        self.info = info;
        self.y_range = None
        self.avg_y_min = None
        self.avg_y_max = None

        EVT_DATA_EVENT (self, self.set_data)
        wx.EVT_CLOSE (self, self.on_close_window)

        self.input_watcher = input_watcher (info.file_descriptor,
                                            info.fft_size,
                                            self)

    def on_close_window (self, event):
        print "fft_window:on_close_window"
        self.keep_running = False
        

    def set_data (self, evt):
        data = evt.data
        dB = 20 * Numeric.log10 (abs(data) + 1e-8)
        l = len (dB)

        if self.info.sample_rate >= 1e6:
            sf = 1e-6
            units = "MHz"
        else:
            sf = 1e-3
            units = "kHz"

        if self.info.input_is_real:     # only plot 1/2 the points
            x_vals = Numeric.arrayrange (l/2) * (self.info.sample_rate * sf / l)
            points = zip (x_vals, dB[0:l/2])
        else:
            # the "negative freqs" are in the second half of the array
            x_vals = Numeric.arrayrange (-l/2+1, l/2) * (self.info.sample_rate * sf / l)
            points = zip (x_vals, Numeric.concatenate ((dB[l/2:], dB[0:l/2])))


        lines = plot.PolyLine (points, colour='LIME GREEN')

        graphics = plot.PlotGraphics ([lines],
                                      title=self.info.title,
                                      xLabel = units, yLabel = "dB")

        self.Draw (graphics, xAxis=None, yAxis=self.y_range)
        self.update_y_range ()

    def update_y_range (self):
        alpha = 1.0/25
        graphics = self.last_draw[0]
        p1, p2 = graphics.boundingBox ()     # min, max points of graphics

        if self.avg_y_min:
            self.avg_y_min = p1[1] * alpha + self.avg_y_min * (1 - alpha)
            self.avg_y_max = p2[1] * alpha + self.avg_y_max * (1 - alpha)
        else:
            self.avg_y_min = p1[1]
            self.avg_y_max = p2[1]

        self.y_range = self._axisInterval ('auto', self.avg_y_min, self.avg_y_max)


# ----------------------------------------------------------------
# Standalone test app
# ----------------------------------------------------------------

class test_app_flow_graph (stdgui.gui_flow_graph):
    def __init__(self, frame, panel, vbox, argv):
        stdgui.gui_flow_graph.__init__ (self, frame, panel, vbox, argv)

        # build our flow graph
        input_rate = 20.0001234e6

        src = gr.sig_source_c (input_rate, gr.GR_SIN_WAVE, 5.75e6, 10e3)
        block, fft_win = make_fft_sink_c (self, panel, "Secret Data", 512, input_rate)
        self.connect (src, block)
        vbox.Add (fft_win, 1, wx.EXPAND)


def main ():
    app = stdgui.stdapp (test_app_flow_graph, "FFT Sink Test App")
    app.MainLoop ()

if __name__ == '__main__':
    main ()

# ----------------------------------------------------------------

==> gr-wxgui-0.2.tar.gz: gr-wxgui-0.2/src/python/gnuradio/wxgui/fftsink.py <==
#!/usr/bin/env python
#
# Copyright 2003,2004 Free Software Foundation, Inc.
# 
# This file is part of GNU Radio
# 
# GNU Radio is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
# 
# GNU Radio is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with GNU Radio; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.
# 

from gnuradio import gr
from gnuradio.wxgui import stdgui
import wx
# import wx.lib.plot as plot
import gnuradio.wxgui.plot as plot
import Numeric
import os
import threading
    


# FIXME this should be rewritten to use hierarchical modules (when they're ready)

# ========================================================================
# returns (block, win).
#   block requires a single input stream of float
#   win is a subclass of wxWindow

def make_fft_sink_f (fg, parent, label, fft_size, input_rate):
    (r_fd, w_fd) = os.pipe ()
    fft_rate = 20

    s2p = gr.serial_to_parallel (gr.sizeof_float, fft_size)
    one_in_n = gr.keep_one_in_n (gr.sizeof_float * fft_size,
                             int (input_rate / fft_size / fft_rate))
    fft = gr.fft_vfc (fft_size, True, True)
    dst = gr.file_descriptor_sink (gr.sizeof_gr_complex * fft_size, w_fd)

    fg.connect (s2p, one_in_n)
    fg.connect (one_in_n, fft)
    fg.connect (fft, dst)

    block = s2p                       # head of pipeline

    win = fft_window (fft_info (r_fd, fft_size, input_rate, True, label), parent)
    
    return (block, win)

# ========================================================================
# returns (block, win).
#   block requires a single input stream of gr_complex
#   win is a subclass of wxWindow

def make_fft_sink_c (fg, parent, label, fft_size, input_rate):
    (r_fd, w_fd) = os.pipe ()
    fft_rate = 20

    s2p = gr.serial_to_parallel (gr.sizeof_gr_complex, fft_size)
    one_in_n = gr.keep_one_in_n (gr.sizeof_gr_complex * fft_size,
                             int (input_rate / fft_size / fft_rate))
    fft = gr.fft_vcc (fft_size, True, True)
    dst = gr.file_descriptor_sink (gr.sizeof_gr_complex * fft_size, w_fd)

    fg.connect (s2p, one_in_n)
    fg.connect (one_in_n, fft)
    fg.connect (fft, dst)

    block = s2p                       # head of pipeline

    win = fft_window (fft_info (r_fd, fft_size, input_rate, False, label), parent)

    return (block, win)

# ------------------------------------------------------------------------

myDATA_EVENT = wx.NewEventType()
EVT_DATA_EVENT = wx.PyEventBinder (myDATA_EVENT, 0)


class DataEvent(wx.PyEvent):
    def __init__(self, data):
        wx.PyEvent.__init__(self)
        self.SetEventType (myDATA_EVENT)
        self.data = data

    def Clone (self): 
        self.__class__ (self.GetId())


class fft_info:
    def __init__ (self, file_descriptor, fft_size, sample_rate, input_is_real, title = "fft"):
        self.file_descriptor = file_descriptor
        self.fft_size = fft_size
        self.sample_rate = sample_rate
        self.input_is_real = input_is_real
        self.title = title;
        
class input_watcher (threading.Thread):
    def __init__ (self, file_descriptor, fft_size, event_receiver, **kwds):
        threading.Thread.__init__ (self, **kwds)
        self.setDaemon (1)
        self.file_descriptor = file_descriptor
        self.fft_size = fft_size
        self.event_receiver = event_receiver
        self.keep_running = True
        self.start ()

    def run (self):
        # print "input_watcher: pid = ", os.getpid ()
        while (self.keep_running):
            s = os.read (self.file_descriptor, gr.sizeof_gr_complex * self.fft_size)
            if not s:
                self.keep_running = False
                break

            complex_data = Numeric.fromstring (s, Numeric.Complex32)
            de = DataEvent (complex_data)
            wx.PostEvent (self.event_receiver, de)
            # print "run: len(complex_data) = ", len(complex_data)
            del de
    

class fft_window (plot.PlotCanvas):
    def __init__ (self, info, parent, id = -1,
                  pos = wx.DefaultPosition, size = wx.DefaultSize,
                  style = wx.DEFAULT_FRAME_STYLE, name = ""):
        plot.PlotCanvas.__init__ (self, parent, id, pos, size, style, name)

        self.SetEnableGrid (True)
        self.SetEnableZoom (True)
        # self.SetBackgroundColour ('black')
        
        self.info = info;
        self.y_range = None
        self.avg_y_min = None
        self.avg_y_max = None

        EVT_DATA_EVENT (self, self.set_data)
        wx.EVT_CLOSE (self, self.on_close_window)

        self.input_watcher = input_watcher (info.file_descriptor,
                                            info.fft_size,
                                            self)

    def on_close_window (self, event):
        print "fft_window:on_close_window"
        self.keep_running = False
        

    def set_data (self, evt):
        data = evt.data
        dB = 20 * Numeric.log10 (abs(data) + 1e-8)
        l = len (dB)

        if self.info.sample_rate >= 1e6:
            sf = 1e-6
            units = "MHz"
        else:
            sf = 1e-3
            units = "kHz"

        if self.info.input_is_real:     # only plot 1/2 the points
            x_vals = Numeric.arrayrange (l/2) * (self.info.sample_rate * sf / l)
            points = zip (x_vals, dB[0:l/2])
        else:
            # the "negative freqs" are in the second half of the array
            x_vals = Numeric.arrayrange (-l/2, l/2-1) * (self.info.sample_rate * sf / l)
            points = zip (x_vals, Numeric.concatenate ((dB[l/2:], dB[0:l/2])))


        lines = plot.PolyLine (points, colour='LIME GREEN')

        graphics = plot.PlotGraphics ([lines],
                                      title=self.info.title,
                                      xLabel = units, yLabel = "dB")

        self.Draw (graphics, xAxis=None, yAxis=self.y_range)
        self.update_y_range ()

#   def update_y_range (self):
#       alpha = 1.0/25
#       graphics = self.last_draw[0]
#       p1, p2 = graphics.boundingBox ()     # min, max points of graphics
#
#       if self.avg_y_min:
#           self.avg_y_min = p1[1] * alpha + self.avg_y_min * (1 - alpha)
#           self.avg_y_max = p2[1] * alpha + self.avg_y_max * (1 - alpha)
#       else:
#           self.avg_y_min = p1[1]
#           self.avg_y_max = p2[1]
#
#       self.y_range = self._axisInterval ('auto', self.avg_y_min, self.avg_y_max)

    def update_y_range (self):
        self.y_range = self._axisInterval ('min', -20, 100)


# ----------------------------------------------------------------
# Standalone test app
# ----------------------------------------------------------------

class test_app_flow_graph (stdgui.gui_flow_graph):
    def __init__(self, frame, panel, vbox, argv):
        stdgui.gui_flow_graph.__init__ (self, frame, panel, vbox, argv)

        # build our flow graph
        input_rate = 20.0001234e6

        src = gr.sig_source_c (input_rate, gr.GR_SIN_WAVE, 5.75e6, 10e3)
        block, fft_win = make_fft_sink_c (self, panel, "Secret Data", 512, input_rate)
        self.connect (src, block)
        vbox.Add (fft_win, 1, wx.EXPAND)


def main ():
    app = stdgui.stdapp (test_app_flow_graph, "FFT Sink Test App")
    app.MainLoop ()

if __name__ == '__main__':
    main ()

# ----------------------------------------------------------------
